@@ -1,4 +1,4 @@
-/* main.c - Last modified: 07-Feb-2019 (kobayasy)
+/* main.c - Last modified: 28-Mar-2019 (kobayasy)
  *
  * Copyright (c) 2018-2019 by Yuichi Kobayashi <kobayasy@kobayasy.com>
  *
@@ -39,7 +39,7 @@
 #include "psync.h"
 #include "popen3.h"
 
-#define VERSION "1.10"
+#define VERSION "1.12"
 #define PROTID 0x01705370  /* 'p', 'S', 'p', 1 */
 #define CONFFILE ".psync.conf"
 #define EXPIRE_DEFAULT 400  /* [days] */
@@ -546,9 +546,7 @@ static int run_local_func(CLIST *clist, void *data) {
                 psync->fdin = param->fdin, psync->fdout = param->fdout;
                 if (g.verbose > 1)
                     psync->info = param->info;
-                status = psync_run(true, psync);
-                if (!ISERR(status))
-                    psync_clean(psync);
+                status = psync_run(PSYNC_MASTER, psync);
             }
             else
                 status = STATUS_NOTREADYREMOTE;
@@ -575,11 +573,21 @@ error:
 static int run_local(int fdin, int fdout, int fderr, pid_t pid, void *data) {
     int status = INT_MIN;
     CLIST *clist = data;
-    RUN_PARAM run_param = {fdout, fdin, -1};
-    INFO_PARAM rinfo_param = {info_func, -1};
+    RUN_PARAM run_param = {
+        .fdin  = fdout,
+        .fdout = fdin,
+        .info  = -1
+    };
+    INFO_PARAM rinfo_param = {
+        .func = info_func,
+        .fd   = -1
+    };
     pthread_t rinfo_tid;
     int info_pipe[2] = {-1, -1};
-    INFO_PARAM linfo_param = {info_func, -1};
+    INFO_PARAM linfo_param = {
+        .func = info_func,
+        .fd   = -1
+    };
     pthread_t linfo_tid;
     size_t length;
 
@@ -667,9 +675,7 @@ static int run_remote(int fdin, int fdout, int fderr, pid_t pid, void *data) {
                 if (!rack) {
                     psync->expire = psync->t - clist->expire * (24*60*60);
                     psync->fdin = fdin, psync->fdout = fdout;
-                    status = psync_run(false, psync);
-                    if (!ISERR(status))
-                        psync_clean(psync);
+                    status = psync_run(PSYNC_SLAVE, psync);
                 }
                 else
                     status = STATUS_NOTREADYREMOTE;
