@@ -1,6 +1,6 @@
-/* psync.c - Last modified: 14-Dec-2018 (kobayasy)
+/* psync.c - Last modified: 29-Mar-2019 (kobayasy)
  *
- * Copyright (c) 2018 by Yuichi Kobayashi <kobayasy@kobayasy.com>
+ * Copyright (c) 2018-2019 by Yuichi Kobayashi <kobayasy@kobayasy.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -45,7 +45,7 @@
 
 #define EXPIRE_DEFAULT (400*24*60*60)  /* [sec] */
 #define BACKUP_DEFAULT   (3*24*60*60)  /* [sec] */
-#define POLL_TIMEOUT 30000  /* [msec] */
+#define POLL_TIMEOUT 100000  /* [msec] */
 #define LOADBUFFER_SIZE (16*1024)  /* [byte] */
 
 #ifndef SYMLINK_MAX
@@ -684,7 +684,7 @@ static int add_deleted_func(FSETS sets, FLIST *flocal, FLIST *flast, void *data)
     return status;
 }
 
-static int get_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+static int make_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
     int status = INT_MIN;
     PRIV *priv = data;
 
@@ -725,6 +725,39 @@ static int get_fsynced_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *dat
     return status;
 }
 
+static int make_fsynced_put_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+    int status = INT_MIN;
+    PRIV *priv = data;
+
+    switch (sets) {
+    case FSETS_1AND2:
+        flocal->st.flags |= fremote->st.flags;
+        if (flocal->st.revision != fremote->st.revision &&
+            flocal->st.mtime != fremote->st.mtime )
+            flocal->st.flags |= FST_UPLD;
+        LIST_DELETE(fremote);
+        free(fremote);
+        LIST_DELETE(flocal);
+        LIST_INSERT_PREV(flocal, &priv->fsynced);
+        break;
+    case FSETS_1NOT2:
+        flocal->st.flags |= FST_UPLD;
+        LIST_DELETE(flocal);
+        LIST_INSERT_PREV(flocal, &priv->fsynced);
+        break;
+    case FSETS_2NOT1:
+        LIST_DELETE(fremote);
+        free(fremote);
+        break;
+    }
+    status = 0;
+    return status;
+}
+
+static int make_fsynced_get_func(FSETS sets, FLIST *flocal, FLIST *fremote, void *data) {
+    return make_fsynced_put_func(sets, fremote, flocal, data);
+}
+
 static int preload(PRIV *priv) {
     int status = INT_MIN;
     intmax_t uploadsize, downloadsize;
@@ -819,7 +852,8 @@ static int upload(PRIV *priv) {
                         goto error;
                     }
                     size -= n;
-                    INFO(priv, "U%+jd\n", progress += n);
+                    progress += n;
+                    INFO(priv, "U%+jd\n", progress);
                 }
                 close(fd), fd = -1;
                 break;
@@ -832,7 +866,8 @@ static int upload(PRIV *priv) {
                     status = ERROR_FUPLD;
                     goto error;
                 }
-                INFO(priv, "U%+jd\n", progress += size);
+                progress += size;
+                INFO(priv, "U%+jd\n", progress);
                 break;
             }
             if (unlink(loadname) == -1) {
@@ -890,7 +925,8 @@ static int download(PRIV *priv) {
                         goto error;
                     }
                     size -= n;
-                    INFO(priv, "D%+jd\n", progress += n);
+                    progress += n;
+                    INFO(priv, "D%+jd\n", progress);
                 }
                 close(fd), fd = -1;
                 if (chmod(loadname, fsynced->st.mode & (S_IRWXU|S_IRWXG|S_IRWXO)) == -1) {
@@ -912,7 +948,8 @@ static int download(PRIV *priv) {
                     status = ERROR_FWRITE;
                     goto error;
                 }
-                INFO(priv, "D%+jd\n", progress += size);
+                progress += size;
+                INFO(priv, "D%+jd\n", progress);
                 break;
             }
             tv[0].tv_sec = fsynced->st.mtime, tv[0].tv_usec = 0;
@@ -1029,101 +1066,6 @@ error:
     return status;
 }
 
-typedef struct {
-    PRIV *priv;
-    int status;
-    pthread_t tid;
-} LOAD_PARAM;
-static void *upload_thread(void *data) {
-    LOAD_PARAM *param = data;
-
-    param->status = upload(param->priv);
-    return NULL;
-}
-static int run(bool master, PRIV *priv) {
-    int status = INT_MIN;
-    LOAD_PARAM upload_param = {priv, INT_MIN};
-
-    ONSTOP(priv->stop, ERROR_STOP);
-    load_fsynced(priv);
-    ONSTOP(priv->stop, ERROR_STOP);
-    status = get_flocal(priv);
-    if (ISERR(status))
-        goto error;
-    ONSTOP(priv->stop, ERROR_STOP);
-    status = each_sets_f(&priv->flocal, &priv->fsynced, add_deleted_func, priv, priv->stop);
-    ONSTOP(priv->stop, ERROR_STOP);
-    if (ISERR(status))
-        goto error;
-    if (master) {
-        status = read_f(true, &priv->fremote, priv->fdin, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status)) {
-            status = ERROR_SDNLD;
-            goto error;
-        }
-        status = each_sets_f(&priv->flocal, &priv->fremote, get_fsynced_func, priv, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status))
-            goto error;
-        status = write_f(true, &priv->fsynced, priv->fdout, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status)) {
-            status = ERROR_SUPLD;
-            goto error;
-        }
-    }
-    else {
-        status = write_f(true, &priv->flocal, priv->fdout, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status)) {
-            status = ERROR_SUPLD;
-            goto error;
-        }
-        status = each_next_f(&priv->flocal, delete_func, NULL, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status))
-            goto error;
-        status = read_f(true, &priv->fsynced, priv->fdin, priv->stop);
-        ONSTOP(priv->stop, ERROR_STOP);
-        if (ISERR(status)) {
-            status = ERROR_SDNLD;
-            goto error;
-        }
-    }
-    status = save_fsynced(priv);
-    if (ISERR(status))
-        goto error;
-    ONSTOP(priv->stop, ERROR_STOP);
-    status = preload(priv);
-    if (ISERR(status))
-        goto error;
-    ONSTOP(priv->stop, ERROR_STOP);
-    if (pthread_create(&upload_param.tid, NULL, upload_thread, &upload_param) != 0) {
-        status = ERROR_SYSTEM;
-        goto error;
-    }
-    status = download(priv);
-    if (ISERR(status))
-        goto error;
-    if (pthread_join(upload_param.tid, NULL) != 0) {
-        status = ERROR_SYSTEM;
-        goto error;
-    }
-    if (ISERR(upload_param.status)) {
-        status = upload_param.status;
-        goto error;
-    }
-    ONSTOP(priv->stop, ERROR_STOP);
-    status = commit(priv);
-    if (ISERR(status))
-        goto error;
-    ONSTOP(priv->stop, ERROR_STOP);
-    status = 0;
-error:
-    return status;
-}
-
 static int clean_r(char *pathname, char *name,
                    volatile sig_atomic_t *stop ) {
     int status = INT_MIN;
@@ -1214,6 +1156,108 @@ error:
     return status;
 }
 
+typedef struct {
+    PRIV *priv;
+    int status;
+    pthread_t tid;
+} LOAD_PARAM;
+static void *upload_thread(void *data) {
+    LOAD_PARAM *param = data;
+
+    param->status = upload(param->priv);
+    return NULL;
+}
+static int run(int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data), PRIV *priv) {
+    int status = INT_MIN;
+    LOAD_PARAM upload_param = {
+        .priv   = priv,
+        .status = INT_MIN
+    };
+
+    ONSTOP(priv->stop, ERROR_STOP);
+    load_fsynced(priv);
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = get_flocal(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = each_sets_f(&priv->flocal, &priv->fsynced, add_deleted_func, priv, priv->stop);
+    ONSTOP(priv->stop, ERROR_STOP);
+    if (ISERR(status))
+        goto error;
+    if (func != NULL) {
+        status = read_f(true, &priv->fremote, priv->fdin, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status)) {
+            status = ERROR_SDNLD;
+            goto error;
+        }
+        status = each_sets_f(&priv->flocal, &priv->fremote, func, priv, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status))
+            goto error;
+        status = write_f(true, &priv->fsynced, priv->fdout, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status)) {
+            status = ERROR_SUPLD;
+            goto error;
+        }
+    }
+    else {
+        status = write_f(true, &priv->flocal, priv->fdout, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status)) {
+            status = ERROR_SUPLD;
+            goto error;
+        }
+        status = each_next_f(&priv->flocal, delete_func, NULL, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status))
+            goto error;
+        status = read_f(true, &priv->fsynced, priv->fdin, priv->stop);
+        ONSTOP(priv->stop, ERROR_STOP);
+        if (ISERR(status)) {
+            status = ERROR_SDNLD;
+            goto error;
+        }
+    }
+    status = save_fsynced(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = preload(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
+    if (pthread_create(&upload_param.tid, NULL, upload_thread, &upload_param) != 0) {
+        status = ERROR_SYSTEM;
+        goto error;
+    }
+    status = download(priv);
+    if (ISERR(status))
+        goto error;
+    if (pthread_join(upload_param.tid, NULL) != 0) {
+        status = ERROR_SYSTEM;
+        goto error;
+    }
+    if (ISERR(upload_param.status)) {
+        status = upload_param.status;
+        goto error;
+    }
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = commit(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = clean(priv);
+    if (ISERR(status))
+        goto error;
+    ONSTOP(priv->stop, ERROR_STOP);
+    status = 0;
+error:
+    return status;
+}
+
 PSYNC *psync_new(const char *dirname,
                  volatile sig_atomic_t *stop ) {
     return (PSYNC *)new_priv(dirname, stop);
@@ -1223,10 +1267,22 @@ void psync_free(PSYNC *psync) {
     free_priv((PRIV *)psync);
 }
 
-int psync_run(bool master, PSYNC *psync) {
-    return run(master, (PRIV *)psync);
-}
+int psync_run(PSYNC_MODE mode, PSYNC *psync) {
+    int (*func)(FSETS sets, FLIST *f1, FLIST *f2, void *data);
 
-int psync_clean(PSYNC *psync) {
-    return clean((PRIV *)psync);
+    switch (mode) {
+    case PSYNC_MASTER:
+        func = make_fsynced_func;
+        break;
+    case PSYNC_MASTER_PUT:
+        func = make_fsynced_put_func;
+        break;
+    case PSYNC_MASTER_GET:
+        func = make_fsynced_get_func;
+        break;
+    case PSYNC_SLAVE:
+    default:
+        func = NULL;
+    }
+    return run(func, (PRIV *)psync);
 }
